// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:wyatt_crud_bloc/src/models/queries/queries_interface.dart';

class QueryParserFirestore implements QueryParserInterface {
  @override
  Query parser(QueryInterface condition, Object query) {
    query as Query;
    if (condition is WhereQueryInterface) {
      switch (condition.type) {
        case WhereQueryType.isEqualTo:
          return query.where(condition.field, isEqualTo: condition.value);
        case WhereQueryType.isNotEqualTo:
          return query.where(condition.field, isNotEqualTo: condition.value);
        case WhereQueryType.isLessThan:
          return query.where(condition.field, isLessThan: condition.value);
        case WhereQueryType.isLessThanOrEqualTo:
          return query.where(
            condition.field,
            isLessThanOrEqualTo: condition.value,
          );
        case WhereQueryType.isGreaterThan:
          return query.where(condition.field, isGreaterThan: condition.value);
        case WhereQueryType.isGreaterThanOrEqualTo:
          return query.where(
            condition.field,
            isGreaterThanOrEqualTo: condition.value,
          );
        case WhereQueryType.arrayContains:
          return query.where(condition.field, arrayContains: condition.value);
        case WhereQueryType.arrayContainsAny:
          return query.where(
            condition.field,
            arrayContainsAny: condition.value as List<Object>,
          );
        case WhereQueryType.whereIn:
          return query.where(
            condition.field,
            whereIn: condition.value as List<Object>,
          );
        case WhereQueryType.whereNotIn:
          return query.where(
            condition.field,
            whereNotIn: condition.value as List<Object>,
          );
        case WhereQueryType.isNull:
          return query.where(condition.field, isNull: condition.value as bool);
      }
    } else if (condition is LimitQueryInterface) {
      return query.limit(condition.limit);
    } else if (condition is OrderByQueryInterface) {
      return query.orderBy(
        condition.field,
        descending: !condition.ascending,
      );
    }
    return query;
  }
}

class WhereQueryFirestore extends WhereQueryInterface {
  WhereQueryFirestore(WhereQueryType type, String field, Object value)
      : super(type, field, value);
}

class LimitQueryFirestore extends LimitQueryInterface {
  LimitQueryFirestore(int limit) : super(limit);
}

class OrderByQueryFirestore extends OrderByQueryInterface {
  OrderByQueryFirestore(String field, {bool ascending = true})
      : super(field, ascending: ascending);
}
