// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

// ignore: one_member_abstracts
abstract class QueryParserInterface {
  Object parser(QueryInterface condition, Object query);
}

abstract class QueryInterface {}

enum WhereQueryType {
  isEqualTo,
  isNotEqualTo,
  isLessThan,
  isLessThanOrEqualTo,
  isGreaterThan,
  isGreaterThanOrEqualTo,
  arrayContains,
  arrayContainsAny,
  whereIn,
  whereNotIn,
  isNull,
}

abstract class WhereQueryInterface extends QueryInterface {
  final WhereQueryType type;
  final String field;
  final Object value;

  WhereQueryInterface(this.type, this.field, this.value);
}

abstract class LimitQueryInterface extends QueryInterface {
  final int limit;

  LimitQueryInterface(this.limit);
}

abstract class OrderByQueryInterface extends QueryInterface {
  final String field;
  final bool ascending;

  OrderByQueryInterface(this.field, {this.ascending = true});
}
