// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/pipeline.dart';

class BodyToJsonMiddleware extends Middleware {
  @override
  String getName() => 'BodyToJson';

  @override
  Future<MiddlewareRequest> onRequest(MiddlewareRequest request) {
    print(
      '${getName()}::OnRequest: transforms body in json if Map then update '
      'headers with right content-type',
    );
    var newReq = request.unfreezedRequest;
    final mutation = {
      'content-type': 'application/json; charset=utf-8',
    };
    if (newReq.body is Map) {
      Map<String, String>? headers = newReq.headers;
      if (headers != null) {
        headers.addAll(mutation);
      } else {
        headers = mutation;
      }
      newReq = newReq.copyWith(body: jsonEncode(newReq.body), headers: headers);
      request.updateUnfreezedRequest(newReq);
    }
    return super.onRequest(request);
  }
}
