// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/models/middleware_response.dart';
import 'package:wyatt_http_client/src/pipeline.dart';

class SimpleLoggerMiddleware extends Middleware {
  @override
  String getName() => 'SimpleLogger';

  @override
  Future<MiddlewareRequest> onRequest(MiddlewareRequest request) {
    print(
      '${getName()}::OnRequest: ${request.httpRequest.method} '
      '${request.httpRequest.url}\n${request.unfreezedRequest.headers}'
      '\n>> ${request.unfreezedRequest.body}',
    );
    return super.onRequest(request);
  }

  @override
  Future<MiddlewareResponse> onResponse(MiddlewareResponse response) async {
    final res = await super.onResponse(response);
    print(
      '${getName()}::OnResponse: ${res.httpResponse.statusCode} -> '
      'received ${res.httpResponse.contentLength} bytes',
    );
    return res;
  }
}
