// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/pipeline.dart';
import 'package:wyatt_http_client/src/utils/protocols.dart';

class UriPrefixMiddleware extends Middleware {
  final Protocols protocol;
  final String? authority;

  UriPrefixMiddleware({
    required this.protocol,
    required this.authority,
  });

  @override
  String getName() => 'UriPrefix';

  @override
  Future<MiddlewareRequest> onRequest(MiddlewareRequest request) {
    final Uri uri =
        Uri.parse('${protocol.scheme}$authority${request.httpRequest.url}');
    print('${getName()}::OnRequest: ${request.httpRequest.url} -> $uri');
    request.updateHttpRequest(url: uri);
    return super.onRequest(request);
  }
}
