// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';
import 'dart:typed_data';

import 'package:http/http.dart';

mixin HeadersTransformer on Client {
  Map<String, String>? headersMutator(
    Object? body,
    Map<String, String>? headers,
  );

  @override
  Future<Response> head(
    Uri url, {
    Map<String, String>? headers,
  }) {
    return super.head(
      url,
      headers: headersMutator(null, headers),
    );
  }

  @override
  Future<Response> get(
    Uri url, {
    Map<String, String>? headers,
  }) {
    return super.get(
      url,
      headers: headersMutator(null, headers),
    );
  }

  @override
  Future<Response> post(
    Uri url, {
    Map<String, String>? headers,
    Object? body,
    Encoding? encoding,
  }) {
    return super.post(
      url,
      headers: headersMutator(body, headers),
      body: body,
      encoding: encoding,
    );
  }

  @override
  Future<Response> put(
    Uri url, {
    Map<String, String>? headers,
    Object? body,
    Encoding? encoding,
  }) {
    return super.put(
      url,
      headers: headersMutator(body, headers),
      body: body,
      encoding: encoding,
    );
  }

  @override
  Future<Response> patch(
    Uri url, {
    Map<String, String>? headers,
    Object? body,
    Encoding? encoding,
  }) {
    return super.patch(
      url,
      headers: headersMutator(body, headers),
      body: body,
      encoding: encoding,
    );
  }

  @override
  Future<Response> delete(
    Uri url, {
    Map<String, String>? headers,
    Object? body,
    Encoding? encoding,
  }) {
    return super.delete(
      url,
      headers: headersMutator(body, headers),
      body: body,
      encoding: encoding,
    );
  }

  @override
  Future<String> read(
    Uri url, {
    Map<String, String>? headers,
  }) {
    return super.read(
      url,
      headers: headersMutator(null, headers),
    );
  }

  @override
  Future<Uint8List> readBytes(
    Uri url, {
    Map<String, String>? headers,
  }) {
    return super.readBytes(
      url,
      headers: headersMutator(null, headers),
    );
  }
}
