// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:http/http.dart';
import 'package:wyatt_http_client/src/implemented_base_client.dart';
import 'package:wyatt_http_client/src/mixins/body_transformer.dart';
import 'package:wyatt_http_client/src/mixins/headers_transformer.dart';
import 'package:wyatt_http_client/src/mixins/request_transformer.dart';
import 'package:wyatt_http_client/src/utils/protocols.dart';
import 'package:wyatt_http_client/src/utils/utils.dart';

class RestClient extends ImplementedBaseClient
    with BodyTransformer, HeadersTransformer, RequestTransformer {
  final Protocols protocol;
  final String? authority;

  RestClient({
    this.protocol = Protocols.https,
    this.authority = '',
    super.inner,
  });

  @override
  Object? bodyMutator(Object? body) {
    print(
      'RestClient::bodyMutator -> encode in json if body is Map: ${body is Map}',
    );
    if (body is Map) {
      return jsonEncode(body);
    }
    return body;
  }

  @override
  Map<String, String>? headersMutator(
    Object? body,
    Map<String, String>? headers,
  ) {
    print(
      'RestClient::headersMutator -> add json content-type if body is Map: ${body is Map}',
    );
    final mutation = {
      'content-type': 'application/json; charset=utf-8',
    };
    if (body is Map) {
      if (headers != null) {
        headers.addAll(mutation);
        return headers;
      } else {
        return mutation;
      }
    }
    return headers;
  }

  @override
  BaseRequest requestMutator(BaseRequest request) {
    print(
      'RestClient::requestMutator -> add prefix path: ${protocol.scheme}$authority',
    );
    final Uri uri = Uri.parse('${protocol.scheme}$authority${request.url}');
    return Utils.copyRequestWith(request, url: uri);
  }
}

// class RestClient extends BaseClient {
//   final Protocols protocol;
//   final String? authority;

//   final Client _inner;

//   RestClient({
//     this.protocol = Protocols.https,
//     this.authority = '',
//     Client? inner,
//   }) : _inner = inner ?? Client();

//   String? forceJson(Object? body) {
//     String? b;
//     if (body != null && body is Map) {
//       b = jsonEncode(body);
//     }
//     return b;
//   }

//   Map<String, String>? forceJsonHeader(
//       Object? body, Map<String, String>? headers,) {
//     final Map<String, String> h = headers ?? {};
//     if (body != null && body is Map) {
//       h['Content-Type'] = 'application/json';
//     }
//     return h;
//   }

//   // @override
//   // Future<Response> post(
//   //   Uri url, {
//   //   Map<String, String>? headers,
//   //   Object? body,
//   //   Encoding? encoding,
//   // }) {
//   //   final b = forceJson(body) ?? body;
//   //   final h = forceJsonHeader(body, headers) ?? headers;
//   //   print(b);
//   //   print(h);
//   //   return super.post(
//   //     url,
//   //     headers: h,
//   //     body: b,
//   //     encoding: encoding,
//   //   );
//   // }

//   @override
//   Future<Response> put(
//     Uri url, {
//     Map<String, String>? headers,
//     Object? body,
//     Encoding? encoding,
//   }) {
//     final b = forceJson(body) ?? body;
//     final h = forceJsonHeader(body, headers) ?? headers;
//     return super.put(
//       url,
//       headers: h,
//       body: b,
//       encoding: encoding,
//     );
//   }

//   @override
//   Future<Response> patch(
//     Uri url, {
//     Map<String, String>? headers,
//     Object? body,
//     Encoding? encoding,
//   }) {
//     final b = forceJson(body) ?? body;
//     final h = forceJsonHeader(body, headers) ?? headers;
//     return super.patch(
//       url,
//       headers: h,
//       body: b,
//       encoding: encoding,
//     );
//   }

//   @override
//   Future<Response> delete(
//     Uri url, {
//     Map<String, String>? headers,
//     Object? body,
//     Encoding? encoding,
//   }) {
//     final b = forceJson(body) ?? body;
//     final h = forceJsonHeader(body, headers) ?? headers;
//     return super.delete(
//       url,
//       headers: h,
//       body: b,
//       encoding: encoding,
//     );
//   }

//   @override
//   Future<StreamedResponse> send(BaseRequest request) {
//     final Uri uri = Uri.parse('${protocol.scheme}$authority${request.url}');
//     return _inner.send(
//       Utils.copyRequestWith(
//         request,
//         url: uri,
//       ),
//     );
//   }
// }
